import 'package:flutter/widgets.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
// ignore: implementation_imports
import 'package:riverpod/src/framework.dart';

// ignore: subtype_of_sealed_class
/// A [ChangeNotifierProvider] for [RestorableProperty]
class RestorableProvider<Notifier extends RestorableProperty?>
    extends ChangeNotifierProvider<Notifier> {
  RestorableProvider(
    Create<Notifier, ChangeNotifierProviderRef<Notifier>> create, {
    required this.restorationId,
    String? name,
    List<ProviderOrFamily>? dependencies,
    Family? from,
    Object? argument,
  }) : super(
          create,
          name: name,
          dependencies: dependencies,
          from: from,
          argument: argument,
        );

  /// {@macro flutter.widgets.widgetsApp.restorationScopeId}
  final String restorationId;

  RestorableOverride<Notifier> overrideWithRestorable(Notifier value) {
    return RestorableOverride<Notifier>(this, value);
  }
}

class RestorableOverride<T extends RestorableProperty?> {
  const RestorableOverride(
    this.provider,
    this.value,
  );

  final RestorableProvider<T> provider;
  final T value;
}
